@@ -1,17 +1,15 @@
-#' Compute the gram matrix
-#'
-#' Take a Matrix object and compute a gram matrix
-#'
-#' Due to numerical error in floating points you can occasionally get
-#' very small negative values.  Thus we check if the minimum is under 0 
-#' and if true, assign elements less than 0 to zero exactly.  This is mostly
-#' so we don't risk numerical instability later by introducing negative numbers of 
-#' any sort.
-#' @param mat a Matrix sparse Document by Term matrix
-#' @export
-
+# Compute the gram matrix
+#
+# Take a Matrix object and compute a gram matrix
+#
+# Due to numerical error in floating points you can occasionally get
+# very small negative values.  Thus we check if the minimum is under 0 
+# and if true, assign elements less than 0 to zero exactly.  This is mostly
+# so we don't risk numerical instability later by introducing negative numbers of 
+# any sort.
+# @param mat A Matrix sparse Document by Term matrix
 gram <- function(mat) {
-  nd <- rowSums(mat)
+  nd <- Matrix::rowSums(mat)
   mat <- mat[nd>=2,] #its undefined if we don't have docs of length 2
   nd <- nd[nd>=2]
   divisor <- nd*(nd-1)
@@ -19,7 +17,7 @@ gram <- function(mat) {
   #Htilde <- mat/sqrt(divisor)
   #Hhat <- diag(colSums(mat/divisor))
   #Q <- crossprod(Htilde) - Hhat
-  Q <- crossprod(mat/sqrt(divisor)) - diag(colSums(mat/divisor))
+  Q <- Matrix::crossprod(mat/sqrt(divisor)) - Matrix::diag(Matrix::colSums(mat/divisor))
   #if(min(Q)<0) Q@x[Q@x < 0 ] <- 0
   return(as.matrix(Q))
 }
@@ -77,19 +75,18 @@ gram.rp <- function(mat, s=.05, p=3000, d.group.size=2000, verbose=TRUE) {
   return(as.matrix(Q/Qnorm))
 }
 
-#' Find Anchor Words
-#'
-#' Take a gram matrix Q and returns K anchors.
-#' 
-#' Does not use any randomness. Notes that anchors are such that you can request
-#' more than you want but its not particularly easy to simply start in the middle
-#' of a process.  One possibility for allowing this would be to return Qbar but
-#' I'm not going to worry about that right now.
-#'
-#' @param Q the gram matrix
-#' @param K the number of desired anchors
-#' @param verbose if TRUE prints a dot to the screen after each anchor
-#' @export
+# Find Anchor Words
+#
+# Take a gram matrix Q and returns K anchors.
+# 
+# Does not use any randomness. Notes that anchors are such that you can request
+# more than you want but its not particularly easy to simply start in the middle
+# of a process.  One possibility for allowing this would be to return Qbar but
+# I'm not going to worry about that right now.
+#
+# @param Q The gram matrix
+# @param K The number of desired anchors
+# @param verbose If TRUE prints a dot to the screen after each anchor
 fastAnchor <- function(Qbar, K, verbose=TRUE) {
   basis <- c()
   rowSquaredSums <- rowSums(Qbar^2) #StabilizedGS
@@ -121,34 +118,38 @@ fastAnchor <- function(Qbar, K, verbose=TRUE) {
   return(basis)
 }
 
-#' RecoverL2
-#'
-#' Recover the topic-word parameters from a set of anchor words using the RecoverL2
-#' procedure of Arora et. al.
-#' 
-#' Using the exponentiated algorithm and an L2 loss identify the optimal convex 
-#' combination of the anchor words which can reconstruct each additional word in the 
-#' matrix.  Transform and return as a beta matrix.
-#'
-#' @param Qbar the row-normalized gram matrix
-#' @param anchor a vector of indices for rows of Q containing anchors
-#' @param wprob the empirical word probabilities used to renorm the mixture weights. 
-#' @param verbose if TRUE prints information as it progresses.
-#' @param ... optional arguments that will be passed to the exponentiated gradient algorithm.
-#' @return 
-#' \item{A}{a matrix of dimension K by V.  This is acturally the transpose of A in Arora et al. and the matrix we call beta.}
-#' @export
-recoverL2 <- function(Qbar, anchor, wprob, verbose=TRUE, ...) {
+# RecoverL2
+#
+# Recover the topic-word parameters from a set of anchor words using the RecoverL2
+# procedure of Arora et. al.
+# 
+# Using the exponentiated algorithm and an L2 loss identify the optimal convex 
+# combination of the anchor words which can reconstruct each additional word in the 
+# matrix.  Transform and return as a beta matrix.
+#
+# @param Qbar The row-normalized gram matrix
+# @param anchor A vector of indices for rows of Q containing anchors
+# @param wprob The empirical word probabilities used to renorm the mixture weights. 
+# @param verbose If TRUE prints information as it progresses.
+# @param ... Optional arguments that will be passed to the exponentiated gradient algorithm.
+# @return 
+# \item{A}{A matrix of dimension K by V.  This is acturally the transpose of A in Arora et al. and the matrix we call beta.}
+recoverL2 <- function(Qbar, anchor, wprob, verbose=TRUE, recoverEG=TRUE, ...) {
   #NB: I've edited the script to remove some of the calculations by commenting them
   #out.  This allows us to store only one copy of Q which is more memory efficient.
   #documentation for other pieces is below.
-  #' \item{R}{a matrix of dimensions K by K that contains the topic covariances.}
-  #' \item{condprob}{a list of exponentiated gradient results.  useful for checking convergence.}
-  
+
   #Qbar <- Q/rowSums(Q)
   X <- Qbar[anchor,]
   XtX <- tcrossprod(X)
   
+  #In a minute we will do quadratic programming
+  #these jointly define the conditions.  First column
+  #is a sum to 1 constraint.  Remainder are each parameter
+  #greater than 0.
+  Amat <- cbind(1,diag(1,nrow=nrow(X)))
+  bvec <- c(1,rep(0,nrow(X)))
+  
   #Word by Word Solve For the Convex Combination
   condprob <- vector(mode="list", length=nrow(Qbar))
   for(i in 1:nrow(Qbar)) {
@@ -156,10 +157,25 @@ recoverL2 <- function(Qbar, anchor, wprob, verbose=TRUE, ...) {
       #if its an anchor we create a dummy entry that is 1 by definition
       vec <- rep(0, nrow(XtX))
       vec[match(i,anchor)] <- 1
-      condprob[[i]] <- list(par=vec)
+      condprob[[i]] <- vec
     } else {
       y <- Qbar[i,]
-      condprob[[i]] <- expgrad(X,y,XtX, ...)
+      
+      if(recoverEG) {
+        solution <- expgrad(X,y,XtX, ...)$par
+      } else {
+        #meq=1 means the sum is treated as an exact equality constraint
+        #and the remainder are >=
+        solution <- quadprog::solve.QP(Dmat=XtX, dvec=X%*%y, 
+                                       Amat=Amat, bvec=bvec, meq=1)$solution  
+      }
+      
+      if(any(solution <= 0)) {
+        #we can get exact 0's or even slightly negative numbers from quadprog
+        #replace with machine double epsilon
+        solution[solution<=0] <- .Machine$double.eps
+      } 
+      condprob[[i]] <- solution
     }
     if(verbose) {
       #if(i%%1 == 0) cat(".")
@@ -171,8 +187,7 @@ recoverL2 <- function(Qbar, anchor, wprob, verbose=TRUE, ...) {
   if(verbose) cat("\n")
   #Recover Beta (A in this notation)
   #  Now we have p(z|w) but we want the inverse
-  weights <- lapply(condprob, function(x) x$par)
-  weights <- do.call(rbind, weights)
+  weights <- do.call(rbind, condprob)
   A <- weights*wprob
   A <- t(A)/colSums(A)
   
@@ -183,32 +198,31 @@ recoverL2 <- function(Qbar, anchor, wprob, verbose=TRUE, ...) {
   #return(list(A=A, R=R, condprob=condprob))
 }
 
-#' Exponentiated Gradient with L2 Loss
-#'
-#' Find the optimal convex combination of features X which approximate the vector y under
-#' and L2 loss.
-#' 
-#' An implementation of RecoverL2 based on David Mimno's code.  In this setting the
-#' objective and gradient can both be kernalized leading to faster computations than
-#' possible under the KL loss.  Specifically the computations are no longer dependent
-#' on the size of the vocabulary making the operation essentially linear on the total
-#' vocab size.
-#' The matrix X'X can be passed as an argument in settings
-#' like spectral algorithms where it is constant across multiple optimizations.  
-#'
-#' @param X the transposed feature matrix.
-#' @param y the target vector
-#' @param XtX optionally a precalculated crossproduct.
-#' @param alpha an optional initialization of the parameter.  Otherwise it starts at 1/nrow(X).
-#' @param tol convergence tolerance
-#' @param max.its maximum iterations to run irrespective of tolerance
-#' @return 
-#' \item{par}{optimal weights}
-#' \item{its}{number of iterations run}
-#' \item{converged}{logical indicating if it converged}
-#' \item{entropy}{entropy of the resulting weights}
-#' \item{log.sse}{log of the sum of squared error}
-#' @export
+# Exponentiated Gradient with L2 Loss
+#
+# Find the optimal convex combination of features X which approximate the vector y under
+# and L2 loss.
+# 
+# An implementation of RecoverL2 based on David Mimno's code.  In this setting the
+# objective and gradient can both be kernalized leading to faster computations than
+# possible under the KL loss.  Specifically the computations are no longer dependent
+# on the size of the vocabulary making the operation essentially linear on the total
+# vocab size.
+# The matrix X'X can be passed as an argument in settings
+# like spectral algorithms where it is constant across multiple optimizations.  
+#
+# @param X The transposed feature matrix.
+# @param y The target vector
+# @param XtX Optionally a precalculated crossproduct.
+# @param alpha An optional initialization of the parameter.  Otherwise it starts at 1/nrow(X).
+# @param tol Convergence tolerance
+# @param max.its Maximum iterations to run irrespective of tolerance
+# @return 
+# \item{par}{Optimal weights}
+# \item{its}{Number of iterations run}
+# \item{converged}{Logical indicating if it converged}
+# \item{entropy}{Entropy of the resulting weights}
+# \item{log.sse}{Log of the sum of squared error}
 expgrad <- function(X, y, XtX=NULL, alpha=NULL, tol=1e-7, max.its=500) {
   if(is.null(alpha)) alpha <- 1/nrow(X) 
   alpha <- matrix(alpha, nrow=1, ncol=nrow(X))
@@ -263,9 +277,24 @@ tsneAnchor <- function(Qbar) {
     stop("Please install the Rtsne and geometry packages to use this setting.")
   } 
   #project to 3-D
-  proj <- Rtsne::Rtsne(Qbar, dims=3) 
+  proj <- try(Rtsne::Rtsne(Qbar, dims=3) , silent=TRUE)
+  if(class(proj)=="try-error") {
+    #if this failed it is probably duplicates which Rtsne cannot handle
+    dup <- duplicated(Qbar)
+    if(!any(dup)) stop("an unknown error has occured in Rtsne")
+    
+    dup <- which(dup)
+    for(r in dup) {
+      row <- Qbar[r,]
+      row[row>0] <- runif(sum(row>0),0,1e-5) # add a bit of noise to non-zero duplicates
+      row <- row/sum(row) #renormalize
+      Qbar[r,] <- row
+    }
+    #and now do it again
+    proj <- Rtsne::Rtsne(Qbar, dims=3) 
+  }
+  
   hull <- geometry::convhulln(proj$Y)
   anchor <- sort(unique(c(hull)))
   return(anchor)
-}
-
+}
\ No newline at end of file
